// @flow
import React from 'react';
import { Icon } from 'semantic-ui-react';

type Props = {
  total: number,
  handlePageChange: Function,
  pageSize: number,
  current?: number,
}
type State = {
  currentPage: number,
  showPrevIcon: boolean,
  showNextIcon: boolean,
}

class Pagination extends React.PureComponent<Props, State> {
  pageTotal: number

  constructor(props: Props) {
    super(props);
    this.state = {
      currentPage: props.current || 1,
      showPrevIcon: false,
      showNextIcon: false,
    };
  }

  componentWillReceiveProps(nextProps: Props) {
    if (nextProps.current !== this.props.current) {
      this.setState({ currentPage: nextProps.current });
    }
  }

  handlePageChange(pageNum: number) {
    let currentPage = pageNum;
    if (currentPage < 1) currentPage = 1;
    if (currentPage > this.pageTotal) currentPage = this.pageTotal;
    if (currentPage !== this.state.currentPage) {
      this.setState({ currentPage });
      if (this.props.handlePageChange) {
        this.props.handlePageChange(currentPage);
      }
    }
  }

  render() {
    const { total, pageSize } = this.props;
    const { showPrevIcon, showNextIcon, currentPage } = this.state;
    const pageTotal = total === 0 ? 1 : Math.ceil(total / pageSize);
    this.pageTotal = pageTotal;
    let [showPrev, showNext] = [];
    let pageTotalArr = Array.from(new Array(pageTotal), (val, index) => index + 1);
    if (pageTotal > 5) {
      [showPrev, showNext] = [true, true];
      if (currentPage < 5) {
        showPrev = false;
        pageTotalArr = [1, 2, 3, 4, 5];
      } else if (currentPage > pageTotal - 4) {
        showNext = false;
        pageTotalArr = [pageTotal - 4, pageTotal - 3, pageTotal - 2, pageTotal - 1, pageTotal];
      } else {
        pageTotalArr = pageTotalArr.filter(pageNum => Math.abs(pageNum - currentPage) < 3);
      }
    }
    return (
      total === 0
        ? false
        : (
          <div className="container">
            <div
              className="page-icon icon-left"
              title="上一页"
              style={currentPage === 1 ? { cursor: 'not-allowed' } : {}}
              onClick={() => { this.handlePageChange(currentPage - 1); }}
              onKeyDown={() => {}}
              role="button"
              tabIndex="-1"
            >
              <Icon name="left chevron" />
            </div>
            {
              pageTotalArr.includes(1)
                ? false
                : (
                  <div
                    className="page-item"
                    onClick={() => { this.handlePageChange(1); }}
                    onKeyDown={() => {}}
                    role="button"
                    tabIndex="-1"
                  >
                  1
                  </div>
                )
            }
            {
              showPrev
                ? (
                  <div
                    className="page-icon page-nav"
                    title="前五页"
                    onMouseEnter={() => { this.setState({ showPrevIcon: true }); }}
                    onMouseLeave={() => { this.setState({ showPrevIcon: false }); }}
                    onClick={() => { this.handlePageChange(currentPage - 5); }}
                    onKeyDown={() => {}}
                    role="button"
                    tabIndex="-1"
                  >
                    {
                  showPrevIcon
                    ? <Icon name="angle double left" />
                    : <span className="page-ellipsis">...</span>
                }
                  </div>)
                : false
            }
            {
              pageTotalArr.map(item => (
                <div
                  className={item === currentPage ? 'page-item page-item-active' : 'page-item'}
                  key={item}
                  onClick={() => { this.handlePageChange(item); }}
                  onKeyDown={() => {}}
                  role="button"
                  tabIndex="-1"
                >
                  {item}
                </div>
              ))
            }
            {
              showNext
                ? (
                  <div
                    className="page-icon page-nav"
                    title="后五页"
                    onMouseEnter={() => { this.setState({ showNextIcon: true }); }}
                    onMouseLeave={() => { this.setState({ showNextIcon: false }); }}
                    onClick={() => { this.handlePageChange(currentPage + 5); }}
                    onKeyDown={() => {}}
                    role="button"
                    tabIndex="-1"
                  >
                    {
                  showNextIcon
                    ? <Icon name="angle double right" />
                    : <span className="page-ellipsis">...</span>
                }
                  </div>
                )
                : false
            }
            {
              pageTotalArr.includes(pageTotal)
                ? false
                : (
                  <div
                    className="page-item"
                    onClick={() => { this.handlePageChange(pageTotal); }}
                    onKeyDown={() => {}}
                    role="button"
                    tabIndex="-1"
                  >
                    {pageTotal}
                  </div>
                )
            }
            <div
              className="page-icon"
              title="下一页"
              style={currentPage === pageTotal ? { cursor: 'not-allowed' } : {}}
              onClick={() => { this.handlePageChange(currentPage + 1); }}
              onKeyDown={() => {}}
              role="button"
              tabIndex="-1"
            >
              <Icon name="right chevron" />
            </div>
            <style jsx>
              {`{
                .container {
                  text-align: right;
                }
              .page-item, .page-icon {
                background: white;
                width: 36px;
                height: 36px;
                line-height: 36px;
                text-align: center;
                border: 1px solid #C8C8C8;
                display: inline-block;
                margin: 0 5px;
                overflow: hidden;
                cursor: pointer;
                user-select:none;
              }
              .page-item-active {
                border: none;
                background: #009edc;
                color: white;
              }
              .page-icon {
                padding-left: 6px;
              }
              .icon-left {
                padding-left: 3px;
              }
              .page-nav {
                 border: none;
                 margin: 0;
                 padding: 0;
              }
              .page-ellipsis {
                position: relative;
                top: -6px;
              }
            }`}
            </style>
          </div>
        )
    );
  }
}
Pagination.defaultProps = {
  current: 1,
};
export default Pagination;
