// @flow
import 'isomorphic-unfetch';
import localforage from 'localforage';
import update from 'immutability-helper';
import Router from 'next/router';

import debug from './debug';

type Params = {
  path: string,
  options: Object
}

function authorization(params: Params, user): Promise<*> {
  const { token } = user;
  let newParams;
  if (token) {
    newParams = update(params, {
      options: {
        headers: {
          $set: Object.assign({}, params.options.headers, {
            Authorization: `Bearer ${token}`,
          }),
        },
      },
    });
  } else {
    newParams = params;
  }

  return Promise.resolve(newParams);
}

export async function handleJsonCode(response: Object) {
  const json = await response.json();

  if (json.code !== 200 && json.code !== 0 && json.code !== undefined) {
    throw new Error(json.message);
  }
  return json;
}

function handleStatus(response: Object, handleRes) {
  const { status } = response;
  let href;
  if (status === 403) { href = '403'; }
  if (handleRes) {
    switch (status) {
      // case 401: href = '403'; break;
      case 403: href = '403'; break;
      case 404: href = '404'; break;
      case 500: href = '500'; break;
      default: href = undefined;
    }
  }
  if (href) {
    Router.push(`/exception?page=${href}`, `/exception/${href}`);
  }
  return response;
}


function handleFetch({ path, options }: Params): Promise<*> {
  return fetch(path, options);
}

export function handleFetchError(e: Error) {
  debug.error(e);
}

function isUnauthorized({ status }: Object) {
  return status === 401;
}

function handleUnauthorized(response) {
  if (isUnauthorized(response)) {
    // FIXME: use redux
    localforage.setItem('user', '');
    localforage.setItem('statusToken401', '401');
    // localStorage.setItem('user_token', '');
    // window.alert('会话过期');
    // const { pathname, search } = window.location;
    // window.location.href = `/login?page=${encodeURIComponent(pathname + search)}`;
  }

  return response;
}

function handleError(response: Object, handleRes: boolean) {
  if (response.status === 504) { throw new Error('连接超时'); }
  if (handleRes) { handleStatus(response, handleRes); }
  if (!response.ok) {
    return response.json().then((err) => {
      throw err;
    });
  }

  return response;
}

const RANGE_PATTERN = /([0-9]+)-([0-9]+)\/([0-9]+)/;


export function getPagingInfo(res: Response) {
  const range = res.headers.get('Content-Range');
  if (!range) { return null; }
  const [, first, last, total] = range.match(RANGE_PATTERN) || [];

  return {
    first: Number(first) || 0,
    last: Number(last) || 0,
    total: Number(total) || 0,
  };
}

export function uniFetchNA(
  path: string,
  options: Object = {},
  handleRes?: boolean = false,
  handleJson?: boolean = true,
): Promise<*> {
  const params = { path, options };
  return handleFetch(params)
    .then(response => handleError(response, handleRes))
    .then(response => handleJsonCode(response, handleJson));
}

export function uniFetch(
  path: string,
  options: Object = {},
  user,
  handleRes?: boolean = false,
  handleJson?: boolean = true,
): Promise<*> {
  const params = { path, options };
  return authorization(params, user)
    .then(handleFetch)
    .then(handleUnauthorized)
    .then(response => handleError(response, handleRes))
    .then(response => handleJsonCode(response, handleJson));
}
